#! /usr/bin/ruby -Ku

$fails = []
def assert(result, expect)
  if result == expect
    STDOUT.print "."
  else
    STDOUT.print "F"
    $fails.push sprintf("failed at %s\n", caller(1)[0])
    $fails.push sprintf("\t%s != %s\n", result.inspect, expect.inspect)
  end
  STDOUT.flush
end

END {
  print "\n"
  for i in $fails
    print i
  end
}

a = "あaいbうcえdお"

assert a, "あaいbうcえdお"
assert a.size, 19
assert a.length, 9
assert sprintf("%x", a[0]), "3042"
assert sprintf("%x", a[1]), "61"

a[1] = 0x3044
assert a, "あいいbうcえdお"
assert a.size, 21
assert a.length, 9

assert a.index("う"), 4
assert a.index(/え/), 6
assert a.rindex("d"), 7
assert a.rindex(/い/),2

assert a.tr("あうお", "auo"), "aいいbucえdo"
assert a.tr("abc", "あうお"), "あいいううおえdお"
assert a.tr("^abcd", "け"), "けけけbけcけdけ"
assert a, "あいいbうcえdお"
assert a.slice!(1,2), "いい"
assert a, "あbうcえdお"

assert "aあbいcう".delete("あ-え", "^い"), "abいc"
assert "aaあああbccdeffいcう".squeeze, "aあbcdefいcう"
assert "aaあああbccdeffいcう".count("あa"), 5
assert "ああいうえおaabbaa".tr_s("あa", "aあ"), "aいうえおあbbあ"
assert "ああいうえおaabbaa".squeeze, "あいうえおaba"


assert "あいa" =~ /い/, 1
assert $', "a"

assert "aiueo"[1,2], "iu"
assert "あいうえお"[1,2], "いう"
assert "aiueo"[3..-1], "eo"
assert "あいうえお"[3..-1], "えお"
assert "あいうえお".sub(/う./, "ud"), "あいudお"
assert "テスト".scan(/./), ["テ", "ス", "ト"]
assert "あかさたなあかさたな".gsub(/な/, "na"), "あかさたnaあかさたna"

assert sprintf("<%.2s>", "あいうえお"), "<あい>"
assert sprintf("<%12s>", "あいうえお"), "<       あいうえお>"
assert sprintf("<%-12s>", "あいうえお"), "<あいうえお       >"

assert "あいうえお".split, ["あいうえお"]
assert "あいうえお".split(//), ["あ", "い", "う", "え", "お"]
assert "あ:い:う:え:お".split(":"), ["あ", "い", "う", "え", "お"]
assert "あ:い:う:え:お".split(/:/), ["あ", "い", "う", "え", "お"]
assert "あ:い:う:え:お".split(/:/, 3), ["あ", "い", "う:え:お"]

assert "あいうえお".reverse, "おえういあ"
assert "aあbいcdef".reverse, "fedcいbあa"

assert "aあbいcdef" =~ /^#/, nil

assert "あいうえお".succ, "あいうえか"
assert "あいうえお".replace("い"), "い"

a = "あaいbうcえdお"
assert a.upcase!, "あAいBうCえDお"
assert a.downcase!, "あaいbうcえdお"

assert a.delete("abcd"), "あいうえお"
assert a.delete("あいうえお"), "abcd"

a = "  あaいbうcえdお  "
assert a.strip, "あaいbうcえdお"

a = "あaいbうcえdおあAいBうCえDお"
assert a.swapcase!, "あAいBうCえDおあaいbうcえdお"

assert a.dump, '"\343\201\202A\343\201\204B\343\201\206C\343\201\210D\343\201\212\343\201\202a\343\201\204b\343\201\206c\343\201\210d\343\201\212"'
assert a.inspect, '"あAいBうCえDおあaいbうcえdお"'
